<?php

function cforge_broadcast_mail_broadcast(&$message, $params) {
  $node = &$params['node'];
  $sender = user_load($node->uid);
  $message['subject'] = t('@site_name: @node_title',
    array('@site_name' => variable_get('site_name'), '@node_title' => $node->title)
  );
  $message['body'][] = t('@name posted a new @type.', array(
    '@name' => $sender->name,
    '@type' => node_type_get_type($node->type)->name
  ));
  $render = node_view($node);
  unset($render['comments']);
  $message['body'][] = drupal_render($render);
  if (variable_get('cforge_broadcast_optout')) {
    $account = user_load_by_mail($message['to']);
    $link = url('user/'. $account->uid. '/account/edit', array('absolute' => TRUE));
    $message['body'][] = t("Opt-out of 'broadcast' messages"). "\n". $link;
  }
  $message['headers']['From'] = $sender->mail;
  $message['headers']['Reply-to'] = $params['replytoinfoat'] ? variable_get('site_mail') : $sender->mail;

  if (isset($node->upload) && is_array($node->upload)) {
    $items = (array)reset($node->upload);
    foreach($items as $att) {
      if (!$att['fid']) continue;
      $file = file_load($att['fid']);
      $message['params']['attachments'][] = array(
        'uri' => $file->uri,
        'filecontent' => file_get_contents($file->uri),
        'filename' => $file->filename,
        'filemime' => file_get_mimetype($file->uri)
      );
    }
  }
}


function _cforge_broadcast_form_cforge_settings_form_alter(&$form, &$form_state) {
  $form['broadcast'] = array(
    '#title' => t('Broadcast settings'),
    '#description' => t('Emails nodes to groups of members.'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    'cforge_broadcast_trader_range' => array(
      '#title' => t('Trader scope'),
      '#description' => t('How far can a normal trader broadcast?'),
      '#type' => 'radios',
      '#options' => array(
        'none' => t('Traders cannot broadcast'),
        'neighbourhood' => t('Only to people in the same neighbourhood'),
        'all' => t('To all online users')
      ),
      '#default_value' => variable_get('cforge_broadcast_trader_range', 'none')
    ),
    'cforge_broadcast_optout' => array(
      '#title' => t('Allow opt-out'),
      '#description' => t('Give each user a setting to opt-out of broadcasts'),
      '#type' => 'checkbox',
      '#default_value' => variable_get('cforge_broadcast_optout', 0)
    )
  );

  $days = array('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
  $months = array('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
  $form['autonews'] = array(
    '#title' => t('Automated news letter'),
    '#description' => t('Sent to all traders in the first hour of the day. Last sent: @date', array('@date' => format_date(variable_get('autonews_last'), 'short'))),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 10,
    //@todo there's no way to put these in order
    'autonews_types' => array(
      '#title' => t('One-line templates'),
      '#description' => t('The tokens (with exclamation marks) will be replaced as the mail is generated. Leave blank to exclude from the mail'),
      '#type' => 'fieldset',
      'autonews_event' => array(
        '#title' => t('Events'),
        '#description' => '!title, !teaser, !date',
        '#type' => 'textfield',
        '#default_value' => variable_get('autonews_event', t('On !date: !title<br />!teaser')),
      ),
      'autonews_user' => array(
        '#title' => t('Members'),
        '#description' => '!name, !aboutme! !locality',
        '#type' => 'textfield',
        '#default_value' => variable_get('autonews_user', t('!name (!locality)<br />!aboutme')),
      ),
      'autonews_news' => array(
        '#title' => t('News'),
        '#description' => '!title, !teaser',
        '#type' => 'textfield',
        '#default_value' => variable_get('autonews_news', t('!title - !teaser')),
      ),
      'autonews_offer' => array(
        '#title' => t('Offers'),
        '#description' => '!title, !teaser, !categories, !user',
        '#type' => 'textfield',
        '#default_value' => variable_get('autonews_offer', t('!title offered by !user <br />!teaser')),
      ),
      'autonews_want' => array(
        '#title' => t('Wants'),
        '#description' => '!title, !teaser, !categories, !user',
        '#type' => 'textfield',
        '#default_value' => variable_get('autonews_want', t('!title wanted by !user<br />!teaser')),
      ),
      'autonews_transaction' => array(
        '#title' => t('Transactions'),
        '#description' => 'uses the sentence display, see admin/accounting/misc',
        '#type' => 'checkbox',
        '#default_value' => variable_get('autonews_transaction', TRUE),
      ),//not photos, not docs. can't be reordered
    ),
    'autonews_day' => array(
      '#title' => t('Day of the week'),
      '#type' => 'radios',
      '#options' => $days,
      '#default_value' => variable_get('autonews_day'),
      '#weight' => 2,
    ),
    'autonews_week' => array(
      '#title' => t('Week(s) of the month'),
      '#type' => 'checkboxes',
      '#options' => array(1 =>1, 2=>2, 3=>3, 4=>4, 5 =>5),
      '#default_value' => variable_get('autonews_week'),
      '#weight' => 3,
    ),
    'autonews_months' => array(
      '#title' => t('Months of the year'),
      '#type' => 'checkboxes',
      '#options' => $months,
      '#default_value' => variable_get('autonews_months'),
      '#weight' => 4,
    )
  );
}

function _cforge_broadcast_form_node_form_alter(&$form, &$form_state) {
  if ($GLOBALS['user']->uid != 1) {
    $form['options'][NODE_BROADCAST_FLAG]['#access'] = FALSE;
  }
  else {
    $form['options'][NODE_BROADCAST_FLAG]['#title'] = t('Has been broadcast');
  }

  if (in_array($form['#node']->type, array('proposition', 'story', 'document', 'event' ))) {
    $form['options']['status']['#description'] = t("Uncheck this to hide this item");
    if (!$form['#node']->{NODE_BROADCAST_FLAG}) {
      $broadcast_options = array(
        '' => '--'. t('Choose').'--',
        'user_chooser_segment_me' => t('Just to me'),
        'user_chooser_segment_roles:'.RID_TRADER => t('All other active traders'),
        'user_chooser_segment_neighbourhood' => t('Everyone in my neighbourhood'),
      );
      //check if and to whom the current user can broadcast, then present the appropriate widget
      if (empty($GLOBALS['user']->roles[RID_COMMITTEE])) {
        switch (variable_get('cforge_broadcast_trader_range', 'none')) {
          case 'none': return;
          case 'all':continue;
          case 'neighbourhood':
            unset($broadcast_options['user_chooser_segment_roles:3']);
        }
      }
      $form['broadcast'] = array(
        '#title' => t('Broadcast'),
        '#description' => t('This can only be broadcast once.'),
        '#type' => 'fieldset',
        '#weight' => 100,
        'audience' => array(
          '#title' => t('Send to'),
          '#type' => 'user_chooser_many',
          '#weight' => 1,
          '#options' => $broadcast_options,
          '#required' => FALSE
        ),
        'replytoinfoat' => array(
          '#title' => t('Reply to: @mail', array('@mail' => variable_get('site_mail'))),
          '#type' => 'checkbox',
          '#access' => user_access('bypass_node_access'),//that means RID_COMMITTEE
          '#weight' => 2,
          '#required' => FALSE
        ),
        'send' => array(
          '#type' => 'submit',
          '#description' => t('Each item can be broadcast only once.'),
          '#weight' => 3,
          '#value' => t('Save & Broadcast'),
          '#submit' => array('node_broadcast_submit'),
          '#attributes' => array('onclick' => 'if(!confirm("'.t('Are you sure you want to broadcast this?').'")){return false;}'),
        ),
      );
      if (count($form['broadcast']['audience']['#options']) == 1) {
        $form['broadcast']['audience']['#value'] = key($form['broadcast']['audience']['#options']);
        $form['broadcast']['send']['#value'] = t(
          'Save & Broadcast to: @whoever',
          array('@whoever' => current($form['broadcast']['audience']['#options']))
        );
      }
      if (isset($form['image'])) {
        $form['broadcast']['#description'] .= ' '.t('Images cannot be included in broadcast mails');
      }
      $form['#validate'][] = 'node_broadcast_validate';
    }
  }
}

function _cforge_broadcast_cron() {
  $queue = DrupalQueue::get('cforge_autonews', TRUE);
  $last = variable_get('autonews_last', strtotime('-1 month'));
  //don't bother if mail was sent already today
  if (date('z') <> date('z', $last)) {
    //should we send a mail on this day of the week?
    if (date('w', REQUEST_TIME) == variable_get('autonews_day')) {
      //should we send a mail on this week of the month?
      $weekNo = date("W") - date("W", strtotime(date("Y-m-01", REQUEST_TIME))) + 1;
      if (in_array($weekNo, variable_get('autonews_week'))) {
        //should we send a mail on this month of the year?
        if (in_array(date('n'), variable_get('autonews_months'))) {
          //time to generate the mail
          $uids = db_select('users', 'u')->fields('u', array('uid'))->condition('status', 1)->condition('login', 0, '>')->execute()->fetchCol();
          foreach ($uids as $uid) {
            $queue->createItem(array('uid' => $uid, 'since' => $last));
          }
          variable_set('autonews_last', REQUEST_TIME);
        }
      }
    }
  }
}


function cforge_broadcast_mail_autonews(&$message, $params) {
  static $content;
  if (!$content) {
    $content = cforge_broadcast_mail_autonews_content($params['since']);
  }
  $message['subject'] = variable_get('autonews_subjectline', t('Latest From @site_name', array('!site_name' => variable_get('site_name'))));
  $message['content'] = $content;
}

function cforge_broadcast_mail_autonews_content($since) {
  static $content;
  if (!$content) {
    $lines = array();
    if ($template = variable_get('autonews_event')) {
      $nids = db_query(
        "SELECT nid from {node} n
          LEFT JOIN {field_data_event_date} d ON n.nid = d.entity_id
          WHERE n.type = 'event'
          AND n.status = 1
          AND d.event_date_value > :now
          AND n.changed > :since",
        array(
          ':now' => REQUEST_TIME,
          ':since' => $since,
        )
      )->fetchCol();
      if ($nids) {
        $lines[] = "\n<p>".t('Events');
        foreach (node_load_multiple($nids) as $node) {
          $more = l(t('More...'), 'node/'.$node->nid, array('absolute' => TRUE));
          $body = field_view_field('node', $node, 'body');
          $body['#label_display'] = 'hidden';
          $text = strip_tags(render($body));
          $date = field_view_field('node', $node, 'event_date');
          $snippet = substr($text, 0, strpos($text, '.')+1)  .' '.$more;
          $lines[] = strtr(
            $template,
            array('!title' => $node->title, '!teaser' => $snippet, '!date' => drupal_render($date[0]))
          );
        }
      }
    }

    if ($template = variable_get('autonews_user')) {
      $uids = db_query(
        "SELECT u.uid FROM {users} u LEFT JOIN {users_roles} ur ON u.uid = ur.uid WHERE ur.rid = :rid AND u.created > :since",
        array(':rid' => RID_TRADER, ':since' => $since)
      )->fetchCol();
      if ($uids) {
        $lines[] = "\n<p>".t('Members');
        foreach (user_load_multiple($uids) as $account) {
          $address = field_get_items('user', $account, 'profile_address');
          $aboutme = field_get_items('user', $account, 'profile_notes');
          $lines[] = strtr(
            $template,
            array('!name' => format_username($account), '!locality' => $address[0]['dependent_locality'], '!aboutme' => $aboutme[0]['value'])
          );
        }
      }
    }

    if ($template = variable_get('autonews_news')) {
      $nids = db_query(
        "SELECT nid from {node} n
          WHERE n.type = 'story'
          AND n.status = 1
          AND n.changed > :since",
        array(
          ':since' => $since,
        )
      )->fetchCol();
      if ($nids) {
        $lines[] = "\n<p>".t('News');
        foreach (node_load_multiple($nids) as $node) {
          $more = l(t('More...'), 'node/'.$node->nid, array('absolute' => TRUE));
          $body = field_view_field('node', $node, 'body');
          $body['#label_display'] = 'hidden';
          $text = strip_tags(render($body));
          $snippet = substr($text, 0, strpos($text, '.')+1) .' '.$more;
          $lines[] = strtr(
            $template,
            array('!title' => $node->title, '!' => $snippet)
          );
        }
      }
    }

    if ($template = variable_get('autonews_offer')) {
      $lines = array_merge($lines, _cforge_broadcast_proposition_lines($template, '1'));
    }
    if ($template = variable_get('autonews_want')) {
      $lines = array_merge($lines, _cforge_broadcast_proposition_lines($template, 0));
    }
    if ($template = variable_get('autonews_transaction')) {
      $serials = db_query(
        "SELECT serial FROM {mcapi_transactions} "
        . "WHERE state > 0 "
        . "AND type NOT IN ('mass', 'auto') "
        . "AND created > :since",
        array(':since' => $since)
      )->fetchCol();
      if ($serials) {
        $lines[] = "\n<p>".t('Transactions');
        $transactions = array();
        foreach (array_unique($serials) as $serial) {
          $transaction = transaction_load($serial);
          $template = str_replace('[transaction:links]', '', variable_get('mcapi_sentence', mcapi_sentence_default()));
          print_r($template);
          $renderable = transaction_view(array($transaction), $template, TRUE);
          $lines[] = "\n".render($renderable);
        }
      }
    }
    print_r($lines);die();
   // $content = implode('<br />', $lines);
  }
  return $lines;
}

function _cforge_broadcast_proposition_lines($template, $want) {
  $nids = db_query(
    "SELECT n.nid from {node} n
      LEFT JOIN {offers_wants} ow ON n.nid = ow.nid
      WHERE n.status = TRUE
      AND n.type = 'proposition'
      AND ow.want = :want
      AND (ow.end > :since1 OR ow.end = 0)
      AND n.changed > :since2",
    array(
      ':since1' => $since,
      ':since2' => $since,
      ':want' => $want
    )
  )->fetchCol();
  $lines = array();
  if ($nids) {
    $lines[] ="\n<p>".($want ? t('Wants') : t('Offers'));
    foreach (node_load_multiple($nids) as $node) {
      $more = l(t('More...'), 'node/'.$node->nid, array('absolute' => TRUE));
      $owner = format_username(user_load($node->uid));
      $body = field_view_field('node', $node, 'body', $node->body);
      $body['#label_display'] = 'hidden';
      $text = strip_tags(render($body));
      $snippet = substr($text, 0, strpos($text, '.')+1) .' '. $more;
      $field = field_view_field('node', $node, 'offers_wants_categories');
      $categories = array();
      foreach (element_children($field) as $child) {
        $categories[] = drupal_render($field[$child]);
      }
      $lines[] = strtr(
        $template,
        array('!title' => $node->title, '!teaser' => $snippet, '!categories' => implode(', ', $categories), '!user' => $owner)
      );
    }
  }
  return $lines;
}
